-------------------------------------------------------------------------------
-- File Author: networkerror
-- File Revision: 42
-- File Date: 2009-11-30T05:33:17Z
--
-- Automatically adds the user to the LFG channel on login and when
-- leaving a group, without interfering with actual use of the LFG tool.
--
-- Send suggestions, comments, and bugs to merial.kilrogg@gmail.com.
-------------------------------------------------------------------------------

AlwaysLFG = LibStub("AceAddon-3.0"):NewAddon("AlwaysLFG", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0");

local Version = "r42";

ROLE_DISABLED = false;
ROLE_ENABLED = true;
NUM_ROLES = 3;
local classRoles = {
--	CLASS = { DPS, TANK, HEALER },
	DRUID = {		ROLE_ENABLED,	ROLE_ENABLED,	ROLE_ENABLED,	},
	PALADIN = {		ROLE_ENABLED,	ROLE_ENABLED,	ROLE_ENABLED,	},
	ROGUE = {		ROLE_ENABLED,	ROLE_DISABLED,	ROLE_DISABLED,	},
	PRIEST = {		ROLE_ENABLED,	ROLE_DISABLED,	ROLE_ENABLED,	},
	WARRIOR = {		ROLE_ENABLED,	ROLE_ENABLED,	ROLE_DISABLED,	},
	HUNTER = {		ROLE_ENABLED,	ROLE_DISABLED,	ROLE_DISABLED,	},
	MAGE = {		ROLE_ENABLED,	ROLE_DISABLED,	ROLE_DISABLED,	},
	WARLOCK = {		ROLE_ENABLED,	ROLE_DISABLED,	ROLE_DISABLED,	},
	SHAMAN = {		ROLE_ENABLED,	ROLE_DISABLED,	ROLE_ENABLED,	},
	DEATHKNIGHT = {	ROLE_ENABLED,	ROLE_ENABLED,	ROLE_DISABLED,	},
}

local Options = {
  type = "group",
  get = function(item) return AlwaysLFG.db.profile[item[#item]] end,
  set = function(item, value) AlwaysLFG.db.profile[item[#item]] = value; if (value == true) then PlaySound("igMainMenuOptionCheckBoxOn") else PlaySound("igMainMenuOptionCheckBoxOff")end; AlwaysLFG:UpdateLFG() end,
  args = {
    leader = {
      type = "toggle",
      desc = "Set Leader option.",
      name = "Leader",
      width = "half",
      order = 1,
    },
    damage = {
      type = "toggle",
      desc = "Set Damage option.",
      name = "Damage",
      width = "half",
      order = 3,
    },
    tank = {
      type = "toggle",
      desc = "Set Tank option.",
      name = "Tank",
      width = "half",
      disabled = function()
      local _, englishClass = UnitClass("player");
      local roleStatus;
        for i=1,NUM_ROLES do
          roleStatus = classRoles[englishClass][2];
          if ( roleStatus == ROLE_DISABLED ) then
            return true
          end
        end
      end,
      order = 3,
    },
    healer = {
      type = "toggle",
      desc = "Set Healer option.",
      name = "Healer",
      width = "half",
      disabled = function()
      local _, englishClass = UnitClass("player");
      local roleStatus;
        for i=1,NUM_ROLES do
          roleStatus = classRoles[englishClass][3];
          if ( roleStatus == ROLE_DISABLED ) then
            return true
          end
        end
      end,
      order = 4,
    },
    blank0 = {
      type = "description",
      name = "",
      width = "full",
      order = 5,
    },
    comment = {
      type = "input",
      desc = "Sets the comment that AlwaysLFG uses in the LFG tool.",
      name = "LFG Comment",
      width = "double",
      order = 6,
    },
    blank1 = {
      type = "description",
      name = "",
      width = "full",
      order = 7,
    },
    auto = {
      type = "toggle",
      desc = "Automatically join LFG channel when leaving groups.",
      name = "Join When Leaving Groups",
      width = "double",
      order = 8,
    },
    blank2 = {
      type = "description",
      name = "",
      width = "full",
      order = 9,
    },
    verbose = {
      type = "toggle",
      desc = "Show console messages when resetting LFG parameters.",
      name = "Verbose",
      width = "double",
      order = 10,
    },
  },
}

local SlashOptions = {
  type = "group",
  handler = AlwaysLFG,
  args = {
    config = {
      type = "execute",
      desc = "Open configuration dialog",
      name = "Configure",
      func = function()
        InterfaceOptionsFrame_OpenToCategory(AlwaysLFG.optionFrames.profiles);
        InterfaceOptionsFrame_OpenToCategory(AlwaysLFG.optionFrames.main);
      end,
    },
    join = {
      type = "execute",
      desc = "Join LFG channel",
      name = "Join LFG",
      func = "JoinLFG",
    },
    leave = {
      type = "execute",
      desc = "Leave LFG channel",
      name = "Leave LFG",
      func = "LeaveLFG",
    },
  },
};

local SlashCmds = {
  "alfg",
  "alwayslfg",
};

local defaults = {
  profile = {
    leader = false,
    damage = true,
    tank = false,
    healer = false,
    comment = "Listening to LFG channel using AlwaysLFG",
    auto = true,
    verbose = false,
  },
};

-- Check whether we're already in LFG/LFM.
function AlwaysLFG:InLFG()
  local _, _, _, _, _, _, _, _, _, queued, lfgStatus, lfmStatus = GetLookingForGroup();
  return (lfgStatus or lfmStatus);
end

-- Update all Blizzard UI elements.
function AlwaysLFG:UpdateLFG()
  LFGFrameLeaderCheckButton:SetChecked(self.db.profile.leader)
  LFGFrameRoleButton2:SetChecked(self.db.profile.tank)
  LFGFrameRoleButton3:SetChecked(self.db.profile.healer)
  LFGFrameRoleButton1:SetChecked(self.db.profile.damage)
  SetLFGRoles(self.db.profile.leader,self.db.profile.tank,self.db.profile.healer,self.db.profile.damage)
  SetLookingForGroup(1, 5, 1)
  SetLFGComment(self.db.profile.comment)
  LFGFrame_UpdateRoleBoxes()
  LFGFrame_UpdateRolesChosen()
  LFMFrame_CacheAndUpdate()
end

-- Add ourselves to LFM if we're not already LFG/LFM.
function AlwaysLFG:JoinLFG()
  if not AlwaysLFG:InLFG() then
    ClearLookingForGroup();
    ClearLookingForMore();
    
    AlwaysLFG:UpdateLFG();
    AlwaysLFG:ScheduleTimer("UpdateLFG", 15)
    
    if self.db.profile.verbose then
      self:Print("Modified LFG settings.");
    end
  end
end

-- Leave the LFG channel.
function AlwaysLFG:LeaveLFG()
  ClearLookingForGroup();
  ClearLookingForMore();

  AlwaysLFG:UpdateLFG();
end


-- Join LFG whenever we leave a group.
function AlwaysLFG:PARTY_MEMBERS_CHANGED()
  local nowInGroup = (GetRealNumPartyMembers() > 0 or GetRealNumRaidMembers() > 0);
  if self.inGroup and not nowInGroup then
    AlwaysLFG:JoinLFG();
  end
  self.inGroup = nowInGroup;
end

-- We're alive!
function AlwaysLFG:OnInitialize()
  -- Load our database.
  self.db = LibStub("AceDB-3.0"):New("AlwaysLFGDB", defaults, "default");

  -- Set up our config options.
  local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db);
  
  local config = LibStub("AceConfig-3.0");
  config:RegisterOptionsTable("AlwaysLFG", SlashOptions, SlashCmds);

  local registry = LibStub("AceConfigRegistry-3.0");
  registry:RegisterOptionsTable("AlwaysLFG Options", Options);
  registry:RegisterOptionsTable("AlwaysLFG Profiles", profiles);

  local dialog = LibStub("AceConfigDialog-3.0");
  self.optionFrames = {
    main     = dialog:AddToBlizOptions("AlwaysLFG Options", "AlwaysLFG"),
    profiles = dialog:AddToBlizOptions("AlwaysLFG Profiles", "Profiles", "AlwaysLFG");
  };

  -- Set initial state.
  self.inGroup = false;

  -- Register for events.
  self:RegisterEvent("PARTY_MEMBERS_CHANGED");

  if self.db.profile.debug then
    self:Print("AlwaysLFG v" .. Version .. " loaded.");
  end
end

-- Fire it up on the first OnUpdate we see
local frame=CreateFrame("Frame")
frame:SetScript("OnUpdate", function(self)
	self:Hide()
    AlwaysLFG:JoinLFG();
end)
frame:Show()
